/*
 * playlistaspect.h
 *
 *  Created on: 15.04.2010
 *      Author: darkstar
 */

#include "playlistaspect.h"

/* PlayListAspectExtension */

void PlayListAspectExtension::execute()
{
	PlayListAspectExtension *next = nextAspectExtension();

	if (next)
		next->execute();
	else
	{
		// We are about to switch the view source table.
		// So, make sure the current order is saved to the table...
		//playListAspect_->ensurePlayListValidInDB();
		//playListAspect_->updateView();
	}
}

void PlayListAspectExtension::reset()
{

}

bool PlayListAspectExtension::isActive()
{
	return true;
}

bool PlayListAspectExtension::isFiltering()
{
	return false;
}

void PlayListAspectExtension::saveExtensionState(QStringList &dst)
{
	// descendants can do work here...
}

void PlayListAspectExtension::loadExtensionState(QStringList &src)
{
	// descendants can do work here...
}

QString PlayListAspectExtension::inputViewSource()
{
	PlayListAspectExtension *prev = previousAspectExtension();

	if (prev)
		return prev->outputViewSource();
	else
		return QString::null;
}

PlayListAspectExtension *PlayListAspectExtension::previousAspectExtension()
{
#ifdef QT4
	int index = extensionList_->indexOf(this);

	if (index > -1 && this != extensionList_->first())
		return extensionList_->at(index - 1);
	else
		return NULL;
#else
	int index = extensionList_->findRef(this);

	if (index > -1 && this != extensionList_->getFirst())
		return extensionList_->at(index - 1);
	else
		return NULL;
#endif
}

PlayListAspectExtension *PlayListAspectExtension::nextAspectExtension()
{
#ifdef QT4
	int index = extensionList_->indexOf(this);

	if (index > -1 && this != extensionList_->last())
		return extensionList_->at(index + 1);
	else
		return NULL;
#else
	int index = extensionList_->findRef(this);

	if (index > -1 && this != extensionList_->getLast())
		return extensionList_->at(index + 1);
	else
		return NULL;
#endif
}

void PlayListAspectExtension::extensionGraphExecutionFinished()
{
	// descendants can do work here...
}

class PlayListAspectFilterExtensionStart: public PlayListAspectExtension
{
public:
	virtual QString outputViewSource()
	{
		return playListAspect()->playList()->sourceTable();
	}
};

/* PlayListAspect */

PlayListAspect::PlayListAspect(PlayList *playList, const QString aspectName)
	:	QObject(),
	 	playList_(playList),
	 	aspectName_(aspectName),
	 	prefix_(playList->sourceTable() + "_" + aspectName + "_")
{
	PlayListAspectFilterExtensionStart *filterExtensionStart = new PlayListAspectFilterExtensionStart();
	registerExtension(filterExtensionStart);

	connect(playList, SIGNAL(playListChanged()), this, SLOT(update()));
}

PlayListAspect::~PlayListAspect()
{

}

void PlayListAspect::update()
{
	extensions_.first()->execute();
}

void PlayListAspect::registerExtension(PlayListAspectExtension *extension)
{
	extensions_.append(extension);
	extension->setPlayListAspect(this);
	extension->setExtensionList(&extensions_);
}

void PlayListAspect::unregisterExtension(PlayListAspectExtension *extension)
{
	extensions_.remove(extension);
	extension->setPlayListAspect(NULL);
	extension->setExtensionList(NULL);
}

void PlayListAspect::loadFromDatabase(const QString &tableName)
{

}

void PlayListAspect::saveToDatabase()
{

}
